/* $Id: pixmap.h,v 1.4 1998/10/25 17:57:16 marcus Exp $
******************************************************************************

   KGIcon - display and drawable definitions

   Copyright (C) 1996	Steffen Seeger	[seeger@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************

   NOTE: NEVER INCLUDE THIS FILE DIRECTLY!

******************************************************************************
*/

typedef struct pixmap
{
	struct display	*dpy;		/* display this is displayed	*/

	struct pixmap	*parent;	/* parent if it is a childmap	*/
	struct pixmap	*next;		/* circular list of brothers	*/
	struct pixmap	*childs;	/* entry to child pixmap list	*/

	kgi_coord	offset;		/* offset from parent origin	*/

	kgi_size	size;		/* size of the (child)map	*/
	kgi_maptype	bpp;		/* buffer layout		*/
	kgi_pixel	*pbuf;		/* pixel value buffer		*/
	kgi_zvalue	*zbuf;		/* z value buffer		*/
	struct fb_ops	*ops;
} pixmap;

typedef struct fb_ops
{
	void	(*move)(pixmap *fb, kgi_uint dst, kgi_uint src, kgi_uint cnt);
	void	(*set) (pixmap *fb, kgi_uint s, pixel val, kgi_uint cnt);
	void	(*invert) (pixmap *fb, kgi_uint start, kgi_uint end);
	void	(*update) (FBARGD, kgi_uint pos, pixel val);
	void	(*flush)  (pixmap *fb);
	void	(*copy_from)(pixmap *fb, void *buf, kgi_uint start, kgi_uint cnt);
	void	(*copy_to)(pixmap *fb, void *buf, kgi_uint start, kgi_uint cnt);

	pixel	(*a_to_p) (pixmap *fb, attr a);
	attr	(*p_to_a) (pixmap *fb, pixel val);
	pixel	(*f_to_p) (pixmap *fb, kgi_uint pos);
	kgi_uint	(*p_to_f) (pixmap *fb, pixel val);

#ifndef __KERNEL__

	void	(*points) 	(FBARGD, kgi_point_data *data, kgi_uint cnt);
	void	(*lines) 	(FBARGD, kgi_line_data  *data, kgi_uint cnt);
	void	(*rectangles)	(FBARGD, kgi_rect_data  *data, kgi_uint cnt);
	void	(*fillrects)	(FBARGD, kgi_rect_data  *data, kgi_uint cnt);
	void	(*arcs)		(FBARGD, kgi_arc_data   *data, kgi_uint cnt);
	void	(*fillarcs)	(FBARGD, kgi_arc_data   *data, kgi_uint cnt);

	void	(*polygon)  (FBARGD, kgi_point_data *data, kgi_uint cnt, int mode);
	void	(*fillpoly) (FBARGD, kgi_point_data *data, kgi_uint cnt, int mode);
#endif
} fb_ops;

#ifdef __KERNEL__
extern inline void fb_move(pixmap *fb, kgi_uint dst, kgi_uint src, kgi_uint cnt)
{ (fb->ops->move)(fb, dst, src, cnt); }

extern inline void fb_memset(pixmap *fb, kgi_uint s, kgi_uint val, kgi_uint cnt)
{ (fb->ops->set)(fb, s, val, cnt); }

extern inline void fb_invert(pixmap *fb, kgi_uint start, kgi_uint end)
{ (fb->ops->invert)(fb, start, end); }

extern inline void fb_update(FBARGD, kgi_uint pos, kgi_uint val)
{ (fb->ops->update)(FBARGC, pos, val); }

extern inline void fb_flush(pixmap *fb)
{ (fb->ops->flush)(fb); }

extern inline void fb_copy_from(pixmap *fb, void *buf, kgi_uint start, kgi_uint cnt)
{ (fb->ops->copy_from)(fb, buf, start, cnt); }

extern inline void fb_copy_to(pixmap *fb, void *buf, kgi_uint start, kgi_uint cnt)
{ (fb->ops->copy_to)(fb, buf, start, cnt); }

extern inline pixel fb_attr_to_pixel(pixmap *fb, attr a)
{ return (fb->ops->a_to_p)(fb, a); }

extern inline attr fb_pixel_to_attr(pixmap *fb, pixel val)
{ return (fb->ops->p_to_a)(fb, val); }

extern inline pixel fb_fpos_to_pixel(pixmap *fb, kgi_uint c)
{ return (fb->ops->f_to_p)(fb, c); }

extern inline kgi_uint fb_pixel_to_fpos(pixmap *fb, pixel val)
{ return (fb->ops->p_to_f)(fb, val); }
#endif
